declare function register(name: string, value: any): void;
declare function unregister(name: string): void;
declare function request(name: string): any;

declare class Model {
    static niw<T = Model>(this: { new(): T }): T
    static find<T = Model>(this: { new(): T }, params?: object): T[]
    static get<T = Model>(this: { new(): T }, params?: object): T

    constructor(options?: { fill?: boolean })
    apply<T = Model>(this: T, model: object): Promise<T>
    save<T = Model>(this: T): Promise<T>
    delete<T = Model>(this: T, options?: {
        preDelete?: boolean,
        postDelete?: boolean,
        beforeCallbacks?: Function[],
        afterCallbacks?: Function[]
    }): Promise<T>
    validate(): Promise<void>
    _validate(): IterableIterator<Error>
}

declare class ModelStore extends Model {
    static get schema(): Record<string, unknown>
    static get idName(): string
}

declare function verify(condition: boolean, message?: string, code?: number, exception?: any, kwargs?: any): void;

declare function load(names?: Array<string>, path?: string, encoding?: string, force?: boolean, ctx?: any): Promise<void>;
declare function conf(name: string, fallback?: any, cast?: string, ctx?: any): any;

declare function getObject(
    params: object,
    options: {
        alias?: boolean,
        page?: boolean,
        find?: boolean,
        norm?: boolean
    }
): object;

declare function initMongo(
    mongoose: unknown,
    uri: string
): Promise<void>;

declare function destroyMongo(
    mongoose: unknown
): void;

export { Model, ModelStore, conf, destroyMongo, getObject, initMongo, load, register, request, unregister, verify };
